#!/usr/bin/perl
#####################################################
# CVSROOT Management Tool
# Author     : Hojin Choi <pynoos@users.kldp.net>
# Last update: 2004/2/12
# $Revision: 1.4 $
# Copyright 2004 by Hojin Choi <pynoos@users.kldp.net>
#####################################################

BEGIN
{
	foreach( 0 .. $#INC ) { undef $INC[$_] if $INC[$_] eq "."; }
}

package cvsnotify;

no lib '.';
use strict 'vars';
use FileHandle;
# Global variables
use vars qw($debug);
use vars qw($debug_logfile);
use vars qw($cvsroot $subroot $subrootenc $module $whoami $mode);
use vars qw(@commit_restricted_users);
use vars qw(%conf %conf_array);
use vars qw(%tpl);
use vars qw($log);
use vars qw(@commitinfo @loginfo);
use vars qw($loginfo_workdir $loginfo_logmsg $loginfo_addeddirectory);
use vars qw(@loginfo_files @loginfo_oldrev @loginfo_newrev);
use vars qw($CVSADMIN);
$CVSADMIN = "CVSADMIN";

sub load_conf($)
{
	my $config = shift;
	my %temp_array;
	open CONF, $config or die "I can't open config [$config]\n";
	while(<CONF>)
	{
		next if /^#/;
		$_ =~ s/\s*$//g;
		next if $_ eq "";

		/(\S+)\s*=\s*(.*)$/;

		my( $key, $value ) = ($1,$2);
		if( $key eq "commitinfo" )
		{
			push @commitinfo, $value;
			next;
		}
		if( $key eq "loginfo" )
		{
			push @loginfo, $value;
			next;
		}
		$conf{$key} = $value;
		push @{$temp_array{$key}}, $value;
	}
	close( CONF );

	# About call to load_conf, the latter, the more prior.
	foreach( keys %temp_array )
	{
		#Place temp_array to lower index to be found early.
		@{$conf_array{$_}} = ( @{$temp_array{$_}}, @{$conf_array{$_}} );
	}
}

sub get_matched_value($$$);

sub load_local_conf($)
{
	my $module = shift;
	my $localconfig;

#Try module dependent including configuration.
	$localconfig = get_matched_value( $module, "include", "Include" );

	if( $localconfig )
	{
		print $log "Try to check out $localconfig\n";
		load_conf( $conf{'cvs'} . " checkout -p \"$localconfig\" 2>>\"$debug_logfile\" |" );
	}
	
#Try current directory configuration.
	$localconfig = "$module/CVSADMIN";
	print $log "Try to check out $localconfig\n";
	load_conf( $conf{'cvs'} . " checkout -p \"$localconfig\" 2>>\"$debug_logfile\" |" );
	unless( $? )
	{
		print $log "Local config load error $localconfig\n";
	}
}

sub check_config()
{
	$conf{'sendmail'} = 'sendmail' unless $conf{'sendmail'};
	$conf{'cvs'} = 'cvs' unless $conf{'cvs'};
	$conf{'diff'} = 'diff' unless $conf{'diff'};
	return 0 unless -x $conf{'sendmail'} && -x $conf{'cvs'} && -x $conf{'diff'};
	return 1;
}

sub replace_template($%)
{
	my ($template, %tplhash) = @_;
	foreach( keys %tplhash )
	{
		$template =~ s/{$_}/$tplhash{$_}/g;
	}
	#print $log $template;
	return $template;
}

sub _sendmail($)
{
	my $content = shift;
	open SENDMAIL, "|" . $conf{'sendmail'} . " -t" or return;
	print SENDMAIL $content;
	close( SENDMAIL );
	if( $? )
	{
		print $log "Mail didn't sent with ". $conf{'sendmail'} . "\n";
	}
	else
	{
		print $log "Mail sent with ". $conf{'sendmail'} . "\n";
	}
	#print $log $content;
}

sub _system($)
{
	$_ = shift;
	print LOG "$_\n";
	system( $_ );
}

sub load_template()
{
	my $tplfile;
	unless($conf{'template'})
	{
		print $log "There is no specified template file, use default cvsnotify-en.tpl\n";
		$tplfile = "cvsnotify-en.tpl";
	}
	else
	{
		$tplfile = $conf{'template'};
	}

	my $tplpath = "$cvsroot/CVSROOT/$tplfile";
	unless( -f $tplpath )
	{
		print "There is no template file [$tplpath]";
		return;
	}
	my $tplkey;
	open TPL, $tplpath or print "I can't open template [$tplpath]\n" and exit( 0 );
	while(<TPL>)
	{
		unless( $tplkey )
		{
			if( /^\s*\[([^\]]+)\]\s*$/ )
			{
				$tplkey = $1;
				print $log "Template key found [$tplkey]\n";
				next;
			}
		}
		else
		{
			if( /^\s*\[\/([^\]]+)\]\s*$/ && $tplkey eq $1)
			{
				chomp $tpl{$tplkey};
				$tplkey = "";
				next;
			}
		}
		next unless $tplkey;
		$tpl{$tplkey} .= $_;
	}
	close( TPL );
}

sub init($)
{
	$cvsroot = $ENV{'CVSROOT'};
	if( substr($cvsroot,0,1) eq ':' )
	{
		print "I can't handle network rooted repository. [$cvsroot]\n";
		return bless {};
	}
	chop $cvsroot if $cvsroot =~ /\/$/;

	$mode = shift;
	$whoami = $ENV{'USER'} || getlogin;

	load_conf( "$cvsroot/CVSROOT/cvsnotify.cf" );

	if( $conf{'debug'} )
	{
		$debug_logfile = "/tmp/cvsnotify-$whoami.log";
	}
	else
	{
		$debug_logfile = "/dev/null";
	}

	$subrootenc = "cvsroot=".$conf{'subroot'}."&" if $conf{'subroot'};

	$log = new FileHandle;
	open $log, ">>$debug_logfile";

	check_config();

	return 0;
}

sub commitinfo(@)
{
	my( @args ) = @_;
	print $log "There is " . ($#commitinfo + 1) . " pre-commit checker(s)\n";
	foreach( @commitinfo )
	{
		my $info = "$cvsroot/CVSROOT/cvsnotify-$_.pl";
		if( -f $info )
		{
			my( $ci ) = $_;
			require $info;
			&{$ci."_preprocess"}(@args);
			foreach( @args )
			{
				print $log "Check.. $ci $_\n";
				&{$ci."_foreach"}($_);
			}
			&{$ci."_postprocess"}(@args);
		}
		else
		{
			print $log "I can't find commitinfo sub-script $info\n";
		}
	}
}

sub get_matched_pattern($$$)
{
	my( $path, $confname, $showtitle ) = @_;
	print "$showtitle: Check $confname\n";
	foreach( @{$conf_array{$confname}} )
	{
		next unless /((.{1})([^\2]*)\2)(.*)\s*$/;
		my( $p, $e ) = ($3, $4 );
		my( $pdesc ) = $1;
		$e =~ s/^\s*(.*)/\1/;
		print $log "Search $confname pattern [$p]\n";
		if( $path =~ /$p/ )
		{
			if( $showtitle )
			{
				print "$showtitle: $pdesc $e\n";
			}
			return $e if $e;
			return $p;
		}
	}
	return "";
}

sub get_matched_value($$$)
{
	my( $path, $confname, $showtitle ) = @_;
	print "$showtitle: Check $confname\n";
	foreach( @{$conf_array{$confname}} )
	{
		next unless /((.{1})([^\2]*)\2)\s+(.*)\s*$/;
		my( $p, $e ) = ($3, $4 );
		my( $pdesc ) = $1;
		print $log "Search $confname pattern [$p]\n";
		if( $path =~ /$p/ )
		{
			if( $showtitle )
			{
				print "$showtitle: $pdesc $e\n";
			}
			return $e;
		}
	}
	return "";
}

sub parse_files_and_revisions($)
{
	my $version_string = shift;
	while( $version_string =~ /(^\s*([^,]+),([0-9.NOE]+),([0-9.NOE]+)\s?)/ )
	{
		print $log "Match: [$1]\n";
		print $log "       file  [$2]\n";
		print $log "       orev. [$3]\n";
		print $log "       nrev. [$4]\n";

		#We use array for keeping sequence
		push @loginfo_files, $2;
		push @loginfo_oldrev, $3;
		push @loginfo_newrev, $4;

		$version_string = substr( $version_string, length($1) );
		print $log "Version string: [$version_string]\n";
	}
}

#Even though we get modules and files from argument splitting, but stdin parsing is more accurate than that
#Because module (path) and files can holds spaces in their names. What a confused environment!

sub extract_loginfo_from_stdin($)
{
#pre-loaded values: $cvsroot

	my $version_string = shift;
	my $message_found = 0;
	while(<STDIN>)
	{
		print $log $_;
		if( /Update of (.*)$/ )
		{
			$module = $1;
			$module =~ s/$cvsroot\///g;
			load_local_conf( $module );
			load_template();
			print $log "Module: [$module]\n";
			$version_string =~ s/^$cvsroot\///;
			$version_string =~ s/^$module\s*//;
			next;
		}
		if( /Log Message/ )
		{
			$message_found = 1;
			next;
		}
		if( /^Status:/ )
		{
			$message_found = 0;
			next;
		}
		if( /In directory[^:]+:(.+)$/ )
		{
			$loginfo_workdir = $1;
			next;
		}
		if( /Directory\s*(.+)\s*added to the repository/ )
		{
			$loginfo_addeddirectory = $1;
			#next;
			#No next the messeage will be found in log message.
		}
		if( $message_found )
		{
			$loginfo_logmsg .= $_;
		}
	}
	close( STDIN );
	parse_files_and_revisions($version_string);
}

sub loginfo()
{
	print $log "There is " . ($#loginfo + 1) . " loginfo checker(s)\n";
	foreach( @loginfo )
	{
		my $info = "$cvsroot/CVSROOT/cvsnotify-$_.pl";
		if( -f $info )
		{
			my( $ci ) = $_;
			require $info;
			&{$ci."_preprocess"}();
			for( 0 .. $#loginfo_files )
			{
				&{$ci."_foreach"}( $loginfo_files[$_], $loginfo_oldrev[$_], $loginfo_newrev[$_]);
			}
			&{$ci."_postprocess"}();
		}
		else
		{
			print $log "I can't find commitinfo sub-script $info\n";
		}
	}
}

sub run(@)
{
	print $log "\n###################### START OF SESSION ############################\n";
	if( $mode eq 'commitinfo' )
	{
		# extract information from argument.
		$module = shift;
		$module =~ s/$cvsroot//g;
		load_local_conf( $module );
		load_template();
		commitinfo(@_);
	}
	elsif( $mode eq 'loginfo' )
	{
#Usually number of arguments from loginfo is just one!
#The calling mechanism is like below
#    cvsnotify-loginfo.pl "module1 src1.c,1.1,1.2 src2.c,1.1.2,1.1.3 src2.h,1.10,1.11"
#the committed files are composed into one argument quoted.

		#extract information from stdard input
		extract_loginfo_from_stdin($_[0]);
		loginfo();
	}
}

1;
